% Simulation der Spulenübertragung in Matlab/Simulink
% ---------------------------------------------------

% Einleitung:
% -----------

% In meinem Rotor-Display habe ich eine induktive Übertragung von Energie
% und Daten vorgenommen. Da zum Zeitpunkt der Erbauung wurde die Auslegung
% rein experimentell vorgenommen. Um den Datendurchsatz zu erhöhen, wurde
% zwischenzeitlich die Frequenz der Spule von 20kHz auf ca. 60kHz
% heraufgesetzt. Dadurch ist es möglich, Daten fehlerfrei mit voller RS232
% Geschwindigkeit (115200 bits/s) unidirektional zu senden. Leider ist
% durch die Frequenzerhöhung auch der primärseitige Leerlaufstrom auf 0,8A
% angestiegen. Im Folgenden versuche ich, ein einfaches, aber trotzdem
% halbwegs genaues Matlab/Simulink-Modell zu entwickeln, um die
% Zusammenhänge und Leistungsgrenzen aufzuzeigen. Ich persönlich werde das
% Modell nutzen, um die Wicklungen meiner Spulen zu optimieren. Vielleicht
% ist ja auch noch eine weitere Steigerung des Datendurchsatzes möglich.
% Jeder, der etwas besser weiß als ich, ist gerne eingeladen, das Modell
% entsprechend abzuändern und Hinweise zu geben. Es wird allen zu Gute
% kommen.

% Grenzen der Modellierung:
% -------------------------

% * Primärseitiger Treiber
%       - Modellierung einer idealen H-Brücke, d.h.
%       - Keine Umschaltzeiten
%       - ideale, rechteckige Flanken
%       - Ausgangsspannung lastunabhängig
%       - Kurz: Ein popeliger Recheckgenerator

% * Trafo
%       - Ein verlustbehafteter, nichtidealer Trafo
%       - d.h. Spulen haben Induktivität, Widerstand
%       - Spulen *haben* Streufluß, also nicht alles kommt auf der anderen
%         Seite an.
%       - Keine Sättigung, Luftspulen (eine gute Annahme, glaub ich!)
%       - Keine Wirbelströme (was bei Luftspulen auch ne gute Annahme ist)

% * Gleichrichtung Sekundärseitig:
%       - Ideale Dioden
%       - d.h. bei U > 0.7 R=0 ansonsten R=0

% Zusammenfassend hoffe ich, dass das Modell möglichst gut die Realität
% trifft.


% Grundgleichungen:
% -----------------

% Hier gibt es coole Infos zum Transformator
% http://de.wikipedia.org/wiki/Transformator

% Ich springe gleich zur Zeile "Der verlustlose Transformator"
% und hole mir folgende Gleichungen heraus:

% U1 = L1 * d_i1/dt + M * d_i2/dt
% U2 = L2 * d_i2/dt + M * d_i1/dt

% Jetzt erweitere ich die Gleichung um den Widerstand der Spule. Widerstand
% R1 ist der Widerstand der ersten (primär-)Spule, R2 der Widerstand der
% anderen. Man kann sich diesen Widerstand einfach in Reihe geschaltet zu
% den beiden Induktionsströmen vorstellen.

% Daraus ergibt sich:

% U1 = L1 * d_i1/dt + M * d_i2/dt + R1 * i1
% U2 = L2 * d_i2/dt + M * d_i1/dt + R2 * i2

% Dann Übertragung in den Laplace Bereich (Ableitungen nach der Zeit werden
% ersetzt durch ein *s) Dadurch kann jetzt mit der Gleichung wie mit einer
% "normalen" Gleichung gerechnet werden

% U1 = L1 * i1*s + M * i2*s + R1 * i1
% U2 = L2 * i2*s + M * i1*s + R2 * i2

% Ich möchte den Trafo als Vierpol modellieren. Also eine "Black Box machen
% bei der man
% primärseitig:     U1 vorgibt und i1 herausbekommt
%                   (Man schließt die Spule an eine Spannungsquelle an)
% sekundärseitig :  U2 herauskommt und man i2 wieder reinsteckt.

% Dies macht das Gleichungssystem zwar unübersichtlicher, ist für mich aber
% irgendwie das logischste. Jetzt werden die Gleichungen so umgestellt,
% dass man sie in Simulink modellieren kann.

% (U1 + R1*i1)*1/s - M*i2 = L1*i1   (primärseitig)
% U2*1/s = L2*i2 +R2*i2*1/s + M*i1  (sekundärseitig)

% Diese Gleichungen finden sich nun im Block "Trafo".


% Daten der Spulen:

% Hier werden die Daten der Spulen abgelegt. Für kurze Luftspulen gilt
% folgende Formel (aus Wikipedia: Zylinderspule)

% L = µ0*N^2*A / (l+0.9*r)

mu0 = 1.2566e-6;     % Magnetische Feldkonstante, nicht ändern!

% Daten primärseitig:

N1 = 25;            % Windungsanzahl primär 
l1 = 0.02;          % Länge der Spule in m
d1 = 0.04;          % Durchmesser der Spule in m
R1 = 0.1;            % Widerstand der Spule in Ohm

L1 = mu0*N1^2*pi*(d1/2)^2 / (l1 + 0.9*d1/2);    % Induktivität berechnet

% Daten sekundärseitig:

N2 = 40;
l2 = 0.018;
d2 = 0.036;
R2 = 0.4;

L2 = mu0*N2^2*pi*(d2/2)^2 / (l2 + 0.9*d2/2);    % Induktivität berechnet


% Nun zu dem Thema Verkoppelung der Spulen. Nur ein Teil des magnetischen
% Flusses geht durch beide Spulen. Ein anderer Teil ist Streufluß und
% leistet nichts. Auf der betreffenden Wikipedia Seite findet sich folgende
% Gleichung:

% k^2 = M^2 / (L1*L2)       Wobei k ein Wert zwischen 0 und 100% annehmen
%                           kann. Die Preisfrage ist, wie gut die Spulen
%                           miteinander verkoppelt sind. Wer weiß hierüber
%                           mehr? ich nehme bei mir einfach mal eine
%                           Kopplung von 70% an.

k = 0.7;                    % Achtung! Werte nur 0 <= k <= 1 !

M = k * sqrt(L1*L2);

% Damit sind die Gleichungen des Transformators vollständig. 

% Gleichrichter:
% --------------

% Sekundärseitig wurde auch noch ein Gleichrichter modelliert. Die Dioden
% schalten erst bei 0,7V durch. Das Problem ist, dass der Transformator in
% unserem Falle eine Stromquelle ist; deswegen sieht der Aufbau etwas
% unintuitiv aus. Ich habe den Gleichrichter aber mal mit verschiedenen
% Strömen und Spannungen getestet; es müsste eigentlich passen.
% Der Gleichrichter wurde noch mit einer internen Rampe und einem fiktiven
% Widerstand versehen. Dies dient nur dazu, die Kennlinie ein wenig zu
% linearisieren. Ohne diesen Widerstand hat Simulink Probleme, die
% numerischen Gleichungen zu lösen. Schön ist es nicht, funktioniert aber.

% Am Ausgang des Gleichrichters ist ein Lastwiderstand angeschlossen, der
% als Ersatz für eine gedachte reale Last stehen soll. Natürlich kann man
% das Modell von dort aus auch noch verfeinern, aber um die groben
% Zusammenhänge zu verstehen ist es meiner Meinung nach nicht notwendig.



% Portpin / Analogkomparator:
% ---------------------------

% Hier wurde ein Portpin mit Hysterese und Diode modelliert. Alternativ
% kann man über den Schalter auch einen Analogkomparator auswählen, der
% keine Hysterese hat und exakt beim Nulldurchgang schaltet.


% Biterkennung:
% -------------

% Dieser Block sieht intern sehr kompliziert aus, die Funktion ist
% allerdings recht einfach. Der Block misst die Zeit von Flanke zu Flanke.
% Wenn die Zeit über einen unter einem vorgegebenen Wert bleibt, wird eine
% logische 1 ausgegeben. Diese bleibt bis zur nächsten Taktflanke stabil.
% Ist die Zeit von Flanke zu Flanke größer, wird eine logische Null
% ausgegeben.
% Auch diese Biterkennung wurde nicht wirklich sauber programmiert, macht
% aber mehr oder weniger was sie soll.

t_threshold = 6.4375e-6;     % Grenzzeit, aber der eine 0 erkannt wird



% Bitgenerator:
% -------------

% Die Bits werden quasi Frequenzmoduliert. Kurze Zeit zwischen zwei Flanken
% sendet eine 1, eine lange Zeit zwischen zwei Flanken sendet eine 0.

t_one = 5.5887e-6;
t_zero = 7.4877e-6;


% Jetzt werden die Daten, die gesendet werden sollen, generiert:
% Dazu wird ein alterierendes Rechtecksignal erzeugt. Die Zeiten zwischen
% den Flanken stellen die Bitwerte dar.
% Der Vektor kann beliebig verlängert oder verkürzt werden und neue Werte
% eingefügt werden. Die Berechnung des Vektors ist hässlich, tut aber ihren
% Dienst. Kann natürlich auch verbessert werden.


Data = [0 0 0 0 1 1 0 1 0 1 1 1 0 0 1 0 1 0 1 1 0 1 1 1 1 0 0 1 0 1 0 1];

    
time = 0;
signal = zeros (length(Data),2);
for i = 1:length(Data)
    signal(i,1) = time;
    if Data(i) == 0
        time = time + t_zero;
    else
        time = time + t_one;
    end
    signal(i,2) = (-1)^i;          % Datenvektor erzeugen
end
    

% Ergebnisse / Diskussion
% -----------------------

% Wird die Simulation so wie sie ist laufen gelassen, so erkennt man am
% Ausgang das Bitmuster, welches in Data vorgegeben wurde. Von dort aus
% kann beliebig an den Parametern herumgedreht werden. Interessanterweise
% funktioniert die Detektierung des Signals über ein Portpin besser als
% über den Komparator. Wird der Komparator über den Schalter in Betrieb
% genommen, so fällt auf, dass er in der Simulation am Anfang eine "1"
% anzeigt, bevor die eigentliche Übertragung losgeht. Meiner Meinung nach
% hat dies nicht viel zu sagen, da das System zu diesem Zeitpunkt ohnehin
% noch nicht richtig eingeschwungen ist.

% Natürlich ist das Modell sehr einfach gehalten, vieles ist unsauber
% geschrieben und gelöst. Dennoch hoffe ich, dass Leute, die auch einmal
% mit einer Spulenübertragung experimentieren wollen, hier gute
% Anhaltspunkte finden. Ich habe die verwendeten Daten, soweit bekannt, aus
% meinem eigenen Rotordisplay verwendet. Die Ergebnisse aus der Simulation
% decken sich auch ganz gut mit meinen realen Erfahrungen.


% Viel Spaß beim Experimentieren und Verbessern!

% 19.1.2008 Stefan Heindel


% Fragen, Anregungen?       Stefan.Heindel@web.de

        




                                
